/*
IBM Confidential

OCO Source Materials

5630-A36

(c) Copyright IBM Corp. 1992, 2002

The source code for this program is not published or other-
wise divested of its trade secrets, irrespective of what has
been deposited with the U.S. Copyright Office.
*/

/*                     Rule-Supplier Exit              JAPIUser.c
 *
 * This module is an example rule-supplier using the Javelin API.
 *
 * Written 5/21/97 by Martin Presler-Marshall
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef NO_MULTICASE
#include "HTAPI.h"
#else
#include "htapi.h"
#endif
 
#include "proxy.h"

/* Strings for tracing; should not be translated. */
static char *Hello =           "JAPIUser: Starting.\n";
static char *ParsingRule =     "JAPIUser: Parsing our rule.\n";
static char *ParsingFailed =   "JAPIUser: Rule parsing failed (blame the gods)!\n";
static char *UsingParsedRule = "JAPIUser: Using preparsed rule.\n";

/* Rules.
 *
 * Note: none of these strings should ever be translated.
 */
char *ExampleRule =
"    (PicsRule-1.0"
"        ("
"            serviceinfo ("
"                        name \"http://www.rsac.org/ratingsv01.html\""
"                        shortname \"RSAC\""
"                        available-with-content \"YES\""
"                        )"
"            name        ("
"                        rulename \"RSAC Example\""
"                        description \"Example rule using the RSAC system to block naughty pictures.\""
"                        )"
"            passURL     (\"http://www.ibm.com/*\") {Always pass the IBM homepage!} "
"            optextension (extension-name \"http://www.ics.raleigh.ibm.com/Javelin/JavelinV1.html\")"
"            javelin-extensions ("
"                               active \"no\""
"                               )"
"            Filter ( Pass '((RSAC.v < 3) && (RSAC.s < 3) && (RSAC.n < 3) && (RSAC.l < 3))' )"
"        )"
"    )";

static int           RuleParsed = 0;
static unsigned long RuleHandle;
static unsigned long RuleLength;

/* Aha, finally.
 * This is the rule-supplier exit. ICS will call this function when the
 * proxy server needs a label to make a filtering decision. If you compile
 * this module to "Labeler.dll", then you'd put the following into the ICS
 * config file (httpd.conf):
 *
 * PicsRuleLookup                     Ruler.dll:RuleSupplier
 *
 * If Ruler.dll isn't in the directory you start ICS from, then put a full
 * pathname on it. On Unices, replace the ".dll" extension with ".o" or ".so",
 * as appropriate.
 */
#define MAX_URL 1020
void HTTPD_LINKAGE RuleSupplier(unsigned char *handle, long *return_code)
{
    long retc;                     /* Stores return code from ICAPI functions. */
    unsigned long l1;

    l1 = (unsigned long) strlen(Hello);
    HTTPD_log_trace(handle, Hello, &l1, &retc);

    /* If we haven't yet parsed our rule, then do so.
     * Note that this code needs to be threadsafed, but for this simple example,
     * we'll ignore that little problem.
     */
    if (!RuleParsed) {
        l1 = (unsigned long) strlen(ParsingRule);
        HTTPD_log_trace(handle, ParsingRule, &l1, &retc);

        RuleLength = strlen(ExampleRule);
        HTTPD_parse_rule(handle, ExampleRule, &RuleLength, &RuleHandle, &retc);
        if (retc != HTTPD_SUCCESS) {
            l1 = (unsigned long) strlen(ParsingFailed);
            HTTPD_log_trace(handle, ParsingFailed, &l1, &retc);

            *return_code = HTTP_SERVER_ERROR;
            goto DropOut;
        }
        RuleParsed = 1;
    }

    l1 = (unsigned long) strlen(UsingParsedRule);
    HTTPD_log_trace(handle, UsingParsedRule, &l1, &retc);

    /* Hand 'em the parsed rule. */
    HTTPD_supply_rule(handle, &RuleHandle, &retc);
    if (retc != HTTPD_SUCCESS) {
        *return_code = HTTP_SERVER_ERROR;
        goto DropOut;
    }

    /* Done! */
    *return_code = HTTP_OK;

DropOut:
    /* And return. ICS will examine the value of "return_code" to tell
     * if we were able to provide a label.
     */

    return;
}

